@@ -1,5 +1,5 @@
 /*
-    clsync - file tree sync utility based on fanotify and inotify
+    clsync - file tree sync utility based on inotify
 
     Copyright (C) 2013  Dmitry Yu Okunev <dyokunev@ut.mephi.ru> 0x8E30679C
 
@@ -24,6 +24,10 @@
 #include <stdint.h>
 #include <sys/types.h>
 
+#ifndef CLSYNC_ITSELF
+#	include <clsync/port-hacks.h>
+#endif
+
 #define CLSYNC_API_VERSION 2
 
 enum eventobjtype {
@@ -31,6 +35,8 @@ enum eventobjtype {
 	EOT_DOESNTEXIST	= 1,		// Doesn't exists (not created yet or already deleted)
 	EOT_FILE	= 2,		// File
 	EOT_DIR		= 3,		// Directory
+
+	// The value cannot be higher than "65535". It's due to recognize_event() function of mon_*.c
 };
 typedef enum eventobjtype eventobjtype_t;
 
