@@ -1,5 +1,5 @@
 /*
-    clsync - file tree sync utility based on fanotify and inotify
+    clsync - file tree sync utility based on inotify
     
     Copyright (C) 2013  Dmitry Yu Okunev <dyokunev@ut.mephi.ru> 0x8E30679C
     
@@ -22,4 +22,5 @@ extern char *fd2fpath_malloc(int fd);
 extern int fileutils_copy(const char *path_from, const char *path_to);
 extern short int fileutils_calcdirlevel(const char *path);
 extern int mkdirat_open(const char *const dir_path, int dirfd_parent, mode_t dir_mode);
+extern uint32_t stat_diff(stat64_t *a, stat64_t *b);
 
