@@ -79,6 +79,8 @@ static const struct option long_options[] =
 	{"pid-file",		required_argument,	NULL,	PIDFILE},
 	{"uid",			required_argument,	NULL,	UID},
 	{"gid",			required_argument,	NULL,	GID},
+	{"privileged-uid",	required_argument,	NULL,	PRIVILEGEDUID},
+	{"privileged-gid",	required_argument,	NULL,	PRIVILEGEDGID},
 	{"sync-handler-uid",	required_argument,	NULL,	SYNCHANDLERUID},
 	{"sync-handler-gid",	required_argument,	NULL,	SYNCHANDLERGID},
 	{"chroot",		required_argument,	NULL,	CHROOT},
@@ -116,7 +118,7 @@ static const struct option long_options[] =
 	{"cgroup-group-name",	required_argument,	NULL,	CG_GROUPNAME},
 #endif
 	{"threading",		required_argument,	NULL,	THREADING},
-	{"retries",		optional_argument,	NULL,	RETRIES},
+	{"retries",		required_argument,	NULL,	RETRIES},
 	{"ignore-failures",	optional_argument,	NULL,	IGNOREFAILURES},
 	{"exit-on-sync-skipping",optional_argument,	NULL,	EXITONSYNCSKIP},
 	{"output",		required_argument,	NULL,	OUTPUT_METHOD},
@@ -144,7 +146,9 @@ static const struct option long_options[] =
 	{"lists-dir",		required_argument,	NULL,	OUTLISTSDIR},
 	{"have-recursive-sync",	optional_argument,	NULL,	HAVERECURSIVESYNC},
 	{"synclist-simplify",	optional_argument,	NULL,	SYNCLISTSIMPLIFY},
+#ifdef AUTORULESW
 	{"auto-add-rules-w",	optional_argument,	NULL,	AUTORULESW},
+#endif
 	{"rsync-inclimit",	required_argument,	NULL,	RSYNCINCLIMIT},
 	{"rsync-prefer-include",optional_argument,	NULL,	RSYNCPREFERINCLUDE},
 	{"ignore-exitcode",	required_argument,	NULL,	IGNOREEXITCODE},
@@ -357,6 +361,7 @@ static char *const status_descr[] = {
 	[STATE_TERM]		= "terminating",
 	[STATE_THREAD_GC]	= "thread gc",
 	[STATE_INITSYNC]	= "initsync",
+	[STATE_HOLDON]		= "hold on",
 	NULL
 };
 
@@ -414,8 +419,10 @@ int parent_isalive() {
 	debug(12, "parent_pid == %u", parent_pid);
 
 	if ((rc=kill(parent_pid, 0))) {
-		debug(1, "kill(%u, 0) => %i", parent_pid, rc);
-		return 0;
+		if (errno == ESRCH) {
+			debug(1, "kill(%u, 0) => %i; errno => %s", parent_pid, rc, strerror(errno));
+			return 0;
+		}
 	}
 
 	return 1;
@@ -476,7 +483,7 @@ pid_t fork_helper() {
 }
 
 int version() {
-	info(PROGRAM" v%i.%i"REVISION"\n\t"AUTHOR"\n\nCompiled with options"
+	info(PROGRAM" v%i.%i.%i"REVISION"\n\t"AUTHOR"\n\nCompiled with options"
 #ifdef _DEBUG_SUPPORT
 		" -D_DEBUG_SUPPORT"
 #endif
@@ -531,7 +538,7 @@ int version() {
 #ifdef HL_LOCKS
 		" -DHL_LOCKS"
 #endif
-		, VERSION_MAJ, VERSION_MIN);
+		, VERSION_MAJ, VERSION_MID, VERSION_MIN);
 	exit(0);
 }
 
@@ -808,15 +815,37 @@ static int synchandler_arg1(char *arg, size_t arg_len, void *_ctx_p) {
 	return synchandler_arg(arg, arg_len, _ctx_p, SHARGS_INITIAL);
 }
 
-int parse_customsignals(ctx_t *ctx_p, char *arg) {
+/* strtol wrapper with error checks */
+static inline long xstrtol(const char *str, int *err) {
+	long res;
+	char *endptr;
+
+	errno = 0;
+	res = strtol(str, &endptr, 0);
+	if (errno || *endptr) {
+		error("argument \"%s\" can't be parsed as a number", str);
+		*err = EINVAL;
+	}
+	return res;
+}
+
+static inline int parse_customsignals(ctx_t *ctx_p, char *arg) {
 	char *ptr = arg, *start = arg;
+	int ret = 0;
 	unsigned int signal;
 	do {
 		switch (*ptr) {
 			case 0:
 			case ',':
 			case ':':
+				// TODO: use xstrtol() instead of atoi()
+
+				//signal = (unsigned int)xstrtol(start, &ret);
 				signal = (unsigned int)atoi(start);
+				if (ret) {
+					errno = ret;
+					return errno;
+				}
 				if (signal == 0) {
 					// flushing the setting
 					int i = 0;
@@ -883,7 +912,8 @@ int parse_customsignals(ctx_t *ctx_p, char *arg) {
 	return 0;
 }
 
-int parse_parameter(ctx_t *ctx_p, uint16_t param_id, char *arg, paramsource_t paramsource) {
+static int parse_parameter(ctx_t *ctx_p, uint16_t param_id, char *arg, paramsource_t paramsource) {
+	int ret = 0;
 #ifdef _DEBUG_FORCE
 	fprintf(stderr, "Force-Debug: parse_parameter(): %i: %i = \"%s\"\n", paramsource, param_id, arg);
 #endif
@@ -962,7 +992,7 @@ int parse_parameter(ctx_t *ctx_p, uint16_t param_id, char *arg, paramsource_t pa
 			ctx_p->flags[param_id]++;
 
 			if (pwd == NULL) {
-				ctx_p->uid = (unsigned int)atol(arg);
+				ctx_p->uid = (unsigned int)xstrtol(arg, &ret);
 				break;
 			}
 
@@ -974,7 +1004,7 @@ int parse_parameter(ctx_t *ctx_p, uint16_t param_id, char *arg, paramsource_t pa
 			ctx_p->flags[param_id]++;
 
 			if (grp == NULL) {
-				ctx_p->gid = (unsigned int)atol(arg);
+				ctx_p->gid = (unsigned int)xstrtol(arg, &ret);
 				break;
 			}
 
@@ -984,9 +1014,18 @@ int parse_parameter(ctx_t *ctx_p, uint16_t param_id, char *arg, paramsource_t pa
 #ifdef CAPABILITIES_SUPPORT
 # ifdef SECCOMP_SUPPORT
 		case SECURESPLITTING: {
-			ctx_p->flags[CHECK_EXECVP_ARGS]++;
-			ctx_p->flags[SECCOMP_FILTER]++;
-			ctx_p->flags[FORBIDDEVICES]++;
+			if (ctx_p->flags_values_raw[CHECK_EXECVP_ARGS] == NULL)
+				ctx_p->flags[CHECK_EXECVP_ARGS]++;
+
+			if (ctx_p->flags_values_raw[SECCOMP_FILTER] == NULL)
+				ctx_p->flags[SECCOMP_FILTER]++;
+
+			if (ctx_p->flags_values_raw[FORBIDDEVICES] == NULL)
+				ctx_p->flags[FORBIDDEVICES]++;
+
+			if (ctx_p->flags_values_raw[SPLITTING] != NULL)
+				break;
+
 			arg = "process";
 		}
 		case SPLITTING: {
@@ -1023,12 +1062,36 @@ int parse_parameter(ctx_t *ctx_p, uint16_t param_id, char *arg, paramsource_t pa
 			break;
 		}
 # endif
+		case PRIVILEGEDUID: {
+			struct passwd *pwd = getpwnam(arg);
+			ctx_p->flags[param_id]++;
+
+			if (pwd == NULL) {
+				ctx_p->privileged_uid = (unsigned int)xstrtol(arg, &ret);
+				break;
+			}
+
+			ctx_p->privileged_uid = pwd->pw_uid;
+			break;
+		}
+		case PRIVILEGEDGID: {
+			struct group *grp = getgrnam(arg);
+			ctx_p->flags[param_id]++;
+
+			if (grp == NULL) {
+				ctx_p->privileged_gid = (unsigned int)xstrtol(arg, &ret);
+				break;
+			}
+
+			ctx_p->privileged_gid = grp->gr_gid;
+			break;
+		}
 		case SYNCHANDLERUID: {
 			struct passwd *pwd = getpwnam(arg);
 			ctx_p->flags[param_id]++;
 
 			if (pwd == NULL) {
-				ctx_p->synchandler_uid = (unsigned int)atol(arg);
+				ctx_p->synchandler_uid = (unsigned int)xstrtol(arg, &ret);
 				break;
 			}
 
@@ -1040,7 +1103,7 @@ int parse_parameter(ctx_t *ctx_p, uint16_t param_id, char *arg, paramsource_t pa
 			ctx_p->flags[param_id]++;
 
 			if (grp == NULL) {
-				ctx_p->synchandler_gid = (unsigned int)atol(arg);
+				ctx_p->synchandler_gid = (unsigned int)xstrtol(arg, &ret);
 				break;
 			}
 
@@ -1233,7 +1296,7 @@ int parse_parameter(ctx_t *ctx_p, uint16_t param_id, char *arg, paramsource_t pa
 			ctx_p->pidfile		= arg;
 			break;
 		case RETRIES:
-			ctx_p->retries		= (unsigned int)atol(arg);
+			ctx_p->retries		= (unsigned int)xstrtol(arg, &ret);
 			break;
 		case THREADING: {
 			char *value, *arg_orig = arg;
@@ -1279,22 +1342,22 @@ int parse_parameter(ctx_t *ctx_p, uint16_t param_id, char *arg, paramsource_t pa
 			ctx_p->cluster_mcastipaddr	= arg;
 			break;
 		case CLUSTERMCASTIPPORT:
-			ctx_p->cluster_mcastipport	= (uint16_t)atoi(arg);
+			ctx_p->cluster_mcastipport	= (uint16_t)xstrtol(arg, &ret);
 			break;
 		case CLUSTERTIMEOUT:
-			ctx_p->cluster_timeout		= (unsigned int)atol(arg);
+			ctx_p->cluster_timeout		= (unsigned int)xstrtol(arg, &ret);
 			break;
 		case CLUSTERNODENAME:
 			ctx_p->cluster_nodename		= arg;
 			break;
 		case CLUSTERHDLMIN:
-			ctx_p->cluster_hash_dl_min	= (uint16_t)atoi(arg);
+			ctx_p->cluster_hash_dl_min	= (uint16_t)xstrtol(arg, &ret);
 			break;
 		case CLUSTERHDLMAX:
-			ctx_p->cluster_hash_dl_max	= (uint16_t)atoi(arg);
+			ctx_p->cluster_hash_dl_max	= (uint16_t)xstrtol(arg, &ret);
 			break;
 		case CLUSTERSDLMAX:
-			ctx_p->cluster_scan_dl_max	= (uint16_t)atoi(arg);
+			ctx_p->cluster_scan_dl_max	= (uint16_t)xstrtol(arg, &ret);
 			break;
 #endif
 		case OUTLISTSDIR:
@@ -1334,16 +1397,16 @@ int parse_parameter(ctx_t *ctx_p, uint16_t param_id, char *arg, paramsource_t pa
 			break;
 		}
 		case SYNCDELAY: 
-			ctx_p->syncdelay		= (unsigned int)atol(arg);
+			ctx_p->syncdelay		= (unsigned int)xstrtol(arg, &ret);
 			break;
 		case DELAY:
-			ctx_p->_queues[QUEUE_NORMAL].collectdelay  = (unsigned int)atol(arg);
+			ctx_p->_queues[QUEUE_NORMAL].collectdelay  = (unsigned int)xstrtol(arg, &ret);
 			break;
 		case BFILEDELAY:
-			ctx_p->_queues[QUEUE_BIGFILE].collectdelay = (unsigned int)atol(arg);
+			ctx_p->_queues[QUEUE_BIGFILE].collectdelay = (unsigned int)xstrtol(arg, &ret);
 			break;
 		case BFILETHRESHOLD:
-			ctx_p->bfilethreshold = (unsigned long)atol(arg);
+			ctx_p->bfilethreshold = (unsigned long)xstrtol(arg, &ret);
 			break;
 		case CANCEL_SYSCALLS: {
 			char *subopts = arg;
@@ -1412,10 +1475,10 @@ int parse_parameter(ctx_t *ctx_p, uint16_t param_id, char *arg, paramsource_t pa
 			break;
 		}
 		case RSYNCINCLIMIT:
-			ctx_p->rsyncinclimit = (unsigned int)atol(arg);
+			ctx_p->rsyncinclimit = (unsigned int)xstrtol(arg, &ret);
 			break;
 		case SYNCTIMEOUT:
-			ctx_p->synctimeout   = (unsigned int)atol(arg);
+			ctx_p->synctimeout   = (unsigned int)xstrtol(arg, &ret);
 			break;
 		case PREEXITHOOK:
 			if (strlen(arg)) {
@@ -1494,7 +1557,19 @@ int parse_parameter(ctx_t *ctx_p, uint16_t param_id, char *arg, paramsource_t pa
 				ctx_p->destproto = NULL;
 			}
 
-			if (sep != NULL) {
+			ctx_p->destdir	 = arg;
+
+			if (sep == NULL) {
+				char *at_ptr = strchr(arg, '@');
+				char *cl_ptr = strchr(arg, ':');
+				if (at_ptr != NULL && cl_ptr != NULL && at_ptr < cl_ptr) {
+					ctx_p->destproto = strdup("rsync+ssh");
+					debug(5, "Destination proto is: %s (case #0)", ctx_p->destproto);
+				}
+				break;
+			}
+
+			{
 				char *ptr = arg;
 				while (ptr < sep) {
 					if (*ptr<'a' || *ptr>'z')
@@ -1507,9 +1582,9 @@ int parse_parameter(ctx_t *ctx_p, uint16_t param_id, char *arg, paramsource_t pa
 					memcpy(ctx_p->destproto, arg, len);
 					ctx_p->destproto[len] = 0;
 				}
+				debug(5, "Destination proto is: %s (case #1)", ctx_p->destproto);
 			}
 
-			ctx_p->destdir	 = arg;
 			break;
 		}
 		case SOCKETPATH:
@@ -1611,13 +1686,13 @@ int parse_parameter(ctx_t *ctx_p, uint16_t param_id, char *arg, paramsource_t pa
 			if (arg == NULL)
 				ctx_p->flags[param_id]++;
 			else
-				ctx_p->flags[param_id] = atoi(arg);
+				ctx_p->flags[param_id] = xstrtol(arg, &ret);
 #ifdef _DEBUG_FORCE
 			fprintf(stderr, "Force-Debug: flag %i is set to %i\n", param_id&0xff, ctx_p->flags[param_id]);
 #endif
 			break;
 	}
-	return 0;
+	return ret;
 }
 
 int arguments_parse(int argc, char *argv[], struct ctx *ctx_p) {
@@ -1731,18 +1806,18 @@ int configs_parse(ctx_t *ctx_p, paramsource_t paramsource) {
 		char **config_path_p = config_paths, *config_path_real = xmalloc(PATH_MAX);
 		size_t config_path_real_size=PATH_MAX;
 
-		char *homedir = getenv("HOME");
-		size_t homedir_len = strlen(homedir);
+		char  *homedir     = getenv("HOME");
+		size_t homedir_len = (homedir == NULL ? 0 :strlen(homedir));
 
-		while(*config_path_p != NULL) {
+		while (*config_path_p != NULL) {
 			size_t config_path_len = strlen(*config_path_p);
 
-			if(config_path_len+homedir_len+3 > config_path_real_size) {
+			if (config_path_len+homedir_len+3 > config_path_real_size) {
 				config_path_real_size = config_path_len+homedir_len+3;
 				config_path_real      = xmalloc(config_path_real_size);
 			}
 
-			if(*config_path_p[0] != '/') {
+			if ((*config_path_p[0] != '/') && (homedir_len >= 0)) {
 				memcpy(config_path_real, homedir, homedir_len);
 				config_path_real[homedir_len] = '/';
 				memcpy(&config_path_real[homedir_len+1], *config_path_p, config_path_len+1);
@@ -1750,7 +1825,7 @@ int configs_parse(ctx_t *ctx_p, paramsource_t paramsource) {
 				memcpy(config_path_real, *config_path_p, config_path_len+1);
 
 			debug(1, "Trying config-file \"%s\" (case #1)", config_path_real);
-			if(!g_key_file_load_from_file(gkf, config_path_real, G_KEY_FILE_NONE, NULL)) {
+			if (!g_key_file_load_from_file(gkf, config_path_real, G_KEY_FILE_NONE, NULL)) {
 				debug(1, "Cannot open/parse file \"%s\"", config_path_real);
 				config_path_p++;
 				continue;
@@ -1974,6 +2049,7 @@ int ctx_check(ctx_t *ctx_p) {
 	)
 		warning("Option \"--rsyncpreferinclude\" is useless if mode is not \"rsyncdirect\", \"rsyncshell\" or \"rsyncso\".");
 
+#ifdef AUTORULESW
 	if (
 		(
 			ctx_p->flags[MODE] == MODE_RSYNCDIRECT ||
@@ -1983,6 +2059,7 @@ int ctx_check(ctx_t *ctx_p) {
 		&& ctx_p->flags[AUTORULESW]
 	)
 		warning("Option \"--auto-add-rules-w\" in modes \"rsyncdirect\", \"rsyncshell\" and \"rsyncso\" may cause unexpected problems.");
+#endif
 
 /*
 	if(ctx_p->flags[HAVERECURSIVESYNC] && (ctx_p->listoutdir == NULL)) {
@@ -2019,6 +2096,14 @@ int ctx_check(ctx_t *ctx_p) {
 		ret = errno = EINVAL;
 		error("Option \"--synclist-simplify\" with nodes \"rsyncdirect\" and \"rsyncshell\" are incompatible.");
 	}
+#ifdef GIO_SUPPORT
+# ifdef SECCOMP_SUPPORT
+	if ((ctx_p->flags[MONITOR] == NE_GIO) && (ctx_p->flags[SECCOMP_FILTER])) {
+		ret = errno = EINVAL;
+		error("GIO is not compatible with seccomp filter (\"--monitor=gio\" and \"--seccomp-filter\" are incompatible)");
+	}
+# endif
+#endif
 
 #ifdef FANOTIFY_SUPPORT
 	if (ctx_p->flags[MONITOR] == NE_FANOTIFY)
@@ -2086,6 +2171,11 @@ int ctx_check(ctx_t *ctx_p) {
 		}
 	}
 
+	if (ctx_p->flags[CHECK_EXECVP_ARGS] && (ctx_p->flags[MODE] == MODE_DIRECT)) {
+		ret = errno = EINVAL;
+		error("Options --check-execvp-arguments/--secure-splitting cannot be used in conjuction with --mode=direct (see \"man 1 clsync\": --check-execvp-arguments).");
+	}
+
 #if 0
 	if (ctx_p->handlerfpath != NULL)
 		if (access(ctx_p->handlerfpath, X_OK) == -1) {
@@ -2295,8 +2385,6 @@ int main(int _argc, char *_argv[]) {
 #ifdef CAPABILITIES_SUPPORT
 	ctx_p->flags[CAP_PRESERVE]		 = CAP_PRESERVE_TRY;
 	ctx_p->caps				 = DEFAULT_PRESERVE_CAPABILITIES;
-	ctx_p->synchandler_uid			 = getuid();
-	ctx_p->synchandler_gid			 = getgid();
 	ctx_p->flags[CAPS_INHERIT]		 = DEFAULT_CAPS_INHERIT;
 	ctx_p->flags[DETACH_IPC]		 = DEFAULT_DETACH_IPC;
 	parse_parameter(ctx_p, LABEL, strdup(DEFAULT_LABEL), PS_DEFAULTS);
@@ -2329,6 +2417,16 @@ int main(int _argc, char *_argv[]) {
 		if(nret) ret = nret;
 	}
 
+	if (!ctx_p->flags[PRIVILEGEDUID])
+		ctx_p->privileged_uid = getuid();
+	if (!ctx_p->flags[PRIVILEGEDGID])
+		ctx_p->privileged_gid = getgid();
+
+	if (!ctx_p->flags[SYNCHANDLERUID])
+		ctx_p->synchandler_uid = ctx_p->privileged_uid;
+	if (!ctx_p->flags[SYNCHANDLERGID])
+		ctx_p->synchandler_gid = ctx_p->privileged_gid;
+
 #ifdef CGROUP_SUPPORT
 	if (ctx_p->cg_groupname == NULL) {
 		ctx_p->cg_groupname = parameter_expand(ctx_p, strdup(DEFAULT_CG_GROUPNAME), 2, NULL, NULL, parameter_get, ctx_p);
@@ -2386,17 +2484,16 @@ int main(int _argc, char *_argv[]) {
 		}
 		debug(5, "rwatchdir == \"%s\"", rwatchdir);
 
-/*
 		stat64_t stat64={0};
 		if (lstat64(ctx_p->watchdir, &stat64)) {
 			error("Cannot lstat64() on \"%s\"", ctx_p->watchdir);
 			if (!ret)
 				ret = errno;
 		} else {
-			if (ctx_p->flags[EXCLUDEMOUNTPOINTS])
-				ctx_p->st_dev = stat64.st_dev;
+			ctx_p->st_dev = stat64.st_dev;
+/*
 			if ((stat64.st_mode & S_IFMT) == S_IFLNK) {
-				// The proplems may be due to FTS_PHYSICAL option of ftp_open() in sync_initialsync_rsync_walk(),
+				// The proplems may be due to FTS_PHYSICAL option of fts_open() in sync_initialsync_rsync_walk(),
 				// so if the "watch dir" is just a symlink it doesn't walk recursivly. For example, in "-R" case
 				// it disables filters, because exclude-list will be empty.
 #ifdef VERYPARANOID
@@ -2415,11 +2512,11 @@ int main(int _argc, char *_argv[]) {
 					ret = EINVAL;
 				} else {
 					char *watchdir_resolved;
-#ifdef VERYPARANOID
+# ifdef PARANOID
 					if (ctx_p->watchdirsize)
 						if (ctx_p->watchdir != NULL)
 							free(ctx_p->watchdir);
-#endif
+# endif
 
 					size_t watchdir_resolved_part_len = strlen(watchdir_resolved_part);
 					ctx_p->watchdirsize = watchdir_resolved_part_len+1;	// Not true for case of relative symlink
@@ -2448,8 +2545,8 @@ int main(int _argc, char *_argv[]) {
 				free(watchdir_resolved_part);
 #endif // VERYPARANOID else
 			}
-		}
 */
+		}
 
 		if (!ret) {
 			parse_parameter(ctx_p, WATCHDIR, rwatchdir, PS_CORRECTION);
@@ -2526,12 +2623,16 @@ int main(int _argc, char *_argv[]) {
 
 	if (ctx_p->handlerfpath != NULL) {
 		char *rhandlerfpath = realpath(ctx_p->handlerfpath, NULL);
+/*
 		if (rhandlerfpath == NULL) {
 			error("Got error while realpath() on \"%s\" [#0].", ctx_p->handlerfpath);
 			ret = errno;
 		}
 		debug(5, "rhandlerfpath == \"%s\"", rhandlerfpath);
-		ctx_p->handlerfpath = rhandlerfpath;
+		ctx_p->handlerfpath = rhandlerfpath;*/
+
+		if (rhandlerfpath != NULL) 
+			ctx_p->handlerfpath = rhandlerfpath;
 
 	}
 
@@ -2749,7 +2850,7 @@ int main(int _argc, char *_argv[]) {
 
 
 #ifdef CAPABILITIES_SUPPORT
-	debug(1, "Preserving Linux capabilites");
+	debug(1, "Preserving Linux capabilities");
 
 	// Tell kernel not clear capabilities when dropping root 
 	if (prctl(PR_SET_KEEPCAPS, 1) < 0) {
@@ -2772,26 +2873,24 @@ int main(int _argc, char *_argv[]) {
 
 	if (ctx_p->flags[GID]) {
 		int rc;
-		debug(3, "Trying to drop gid to %i", ctx_p->gid);
-		rc = setgid(ctx_p->gid);
+		debug(3, "Trying to drop effective gid to %i", ctx_p->gid);
+		rc = setegid(ctx_p->gid);
 
 		if (rc && (ctx_p->flags[GID] != UGID_PRESERVE)) {
-			error("Cannot setgid(%u)", ctx_p->gid);
+			error("Cannot setegid(%u)", ctx_p->gid);
 			ret = errno;
 		}
-		if (!rc) debug(4, "success");
 	}
 
 	if (ctx_p->flags[UID]) {
 		int rc;
-		debug(3, "Trying to drop uid to %i", ctx_p->uid);
-		rc = setuid(ctx_p->uid);
+		debug(3, "Trying to drop effective uid to %i", ctx_p->uid);
+		rc = seteuid(ctx_p->uid);
 
 		if (rc && (ctx_p->flags[UID] != UGID_PRESERVE)) {
-			error("Cannot setuid(%u)", ctx_p->uid);
+			error("Cannot seteuid(%u)", ctx_p->uid);
 			ret = errno;
 		}
-		if (!rc) debug(4, "success");
 	}
 
 	if (main_statusfile_f == NULL && ctx_p->statusfile != NULL) {
@@ -2869,12 +2968,35 @@ int main(int _argc, char *_argv[]) {
 	if (ctx_p->pidfile != NULL) {
 		debug(2, "Trying to open the pidfile \"%s\"", ctx_p->pidfile);
 		pid_t pid = getpid();
+
 		FILE *pidfile = fopen(ctx_p->pidfile, "w");
 		if (pidfile == NULL) {
-			error("Cannot open file \"%s\" to write a pid there",
-				ctx_p->pidfile);
-			ret = errno;
-		} else {
+			// If error
+			if (errno == EACCES) {
+				int fd;
+				uid_t euid = geteuid();
+				gid_t egid = getegid();
+
+				debug(1, "Don't have permissions to open file \"%s\". Trying seteuid(0)+open()+fchown()+close()+seteuid(%i)", ctx_p->pidfile, euid);
+
+				errno  = 0;
+				if (!errno) SAFE ( seteuid(0),							ret = errno );
+				if (!errno) SAFE ( (fd = open(ctx_p->pidfile, O_CREAT|O_WRONLY, 0644)) == -1,	ret = errno );
+				if (!errno) SAFE ( fchown(fd, euid, egid),					ret = errno );
+				if (!errno) SAFE ( close(fd),							ret = errno );
+				if (!errno) SAFE ( seteuid(euid),						ret = errno );
+
+				pidfile = fopen(ctx_p->pidfile, "w");
+			}
+
+			if (pidfile == NULL) {
+				error("Cannot open file \"%s\" to write a pid there",
+					ctx_p->pidfile);
+				ret = errno;
+			}
+		}
+
+		if (pidfile != NULL) {
 			if (fprintf(pidfile, "%u", pid) < 0) {
 				error("Cannot write pid into file \"%s\"",
 					ctx_p->pidfile);
@@ -2893,9 +3015,15 @@ int main(int _argc, char *_argv[]) {
 
 	if (ctx_p->pidfile != NULL) {
 		if (unlink(ctx_p->pidfile)) {
-			error("Cannot unlink pidfile \"%s\"",
-				ctx_p->pidfile);
-			ret = errno;
+			FILE *pidfile;
+
+			debug(1, "Cannot unlink pidfile \"%s\": %s. Just truncating the file.",
+				ctx_p->pidfile, strerror(errno));
+
+			SAFE ( (pidfile = fopen(ctx_p->pidfile, "w")) == NULL,	ret = errno );
+
+			if (pidfile != NULL)
+				fclose(pidfile);
 		}
 	}
 
