@@ -17,8 +17,23 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include <sys/types.h>
 
-void *xmalloc(size_t size);
-void *xcalloc(size_t nmemb, size_t size);
-void *xrealloc(void *oldptr, size_t size);
+extern void *xmalloc(size_t size);
+extern void *xcalloc(size_t nmemb, size_t size);
+extern void *xrealloc(void *oldptr, size_t size);
+#ifdef CAPABILITIES_SUPPORT
+extern void *malloc_align(size_t size);
+extern void *calloc_align(size_t nmemb, size_t size);
+extern char *strdup_protect(const char *src, int prot);
+# ifdef SECCOMP_SUPPORT
+extern int is_protected(void *addr);
+# endif
+#endif
+extern void *shm_malloc(size_t size);
+extern void *shm_malloc_try(size_t size);
+extern void *shm_calloc(size_t nmemb, size_t size);
+extern void shm_free(void *ptr);
+
+extern int memory_init();
 
