@@ -59,7 +59,7 @@
 #include "privileged.h"
 
 #ifdef SECCOMP_SUPPORT
-# include <seccomp.h>			// __NR_*
+# include <syscall.h>			// __NR_*
 # include <sys/prctl.h>			// prctl()
 # include <linux/filter.h>		// struct sock_filter
 # include <linux/seccomp.h>		// SECCOMP_RET_*
@@ -106,6 +106,12 @@
 	SECCOMP_ALLOW_ACCUM_SYSCALL(rt_sigprocmask),			\
 	SECCOMP_ALLOW_ACCUM_SYSCALL(rt_sigaction),			\
 	SECCOMP_ALLOW_ACCUM_SYSCALL(nanosleep),				\
+	SECCOMP_ALLOW_ACCUM_SYSCALL(shmdt),				\
+	SECCOMP_ALLOW_ACCUM_SYSCALL(clone),		/* for --threading */		\
+	SECCOMP_ALLOW_ACCUM_SYSCALL(set_robust_list),	/* for --threading? */		\
+	SECCOMP_ALLOW_ACCUM_SYSCALL(madvise),				\
+	SECCOMP_ALLOW_ACCUM_SYSCALL(exit),				\
+
 
 
 /* Syscalls allowed to non-privileged thread */
@@ -132,6 +138,8 @@ int nonprivileged_seccomp_init(ctx_t *ctx_p) {
 		.filter = filter_w_mprotect_table,
 	};
 
+	debug(5, "enabling the seccomp");
+
 	filter_p = (ctx_p->flags[PERMIT_MPROTECT] ? &filter_w_mprotect : &filter);
 
 	SAFE (prctl(PR_SET_NO_NEW_PRIVS, 1, 0, 0, 0),			return -1);
@@ -142,7 +150,7 @@ int nonprivileged_seccomp_init(ctx_t *ctx_p) {
 #endif
 
 int (*privileged_fork_execvp)(const char *file, char *const argv[]);
-int (*privileged_kill_child)(pid_t pid, int sig);
+int (*privileged_kill_child)(pid_t pid, int sig, char ignoreerrors);
 
 #ifdef CAPABILITIES_SUPPORT
 pid_t		helper_pid = 0;
@@ -218,6 +226,7 @@ struct pa_fork_execvp_arg {
 struct pa_kill_child_arg {
 	pid_t pid;
 	int   signal;
+	char  ignoreerrors;
 };
 
 struct pa_waitpid_arg {
@@ -386,7 +395,7 @@ int cap_enable(__u32 caps) {
 
 	cap_hdr.version = _LINUX_CAPABILITY_VERSION;
 	if (capget(&cap_hdr, &cap_dat) < 0) {
-		error("Cannot get capabilites with capget()");
+		error("Cannot get capabilities with capget()");
 		return errno;
 	}
 
@@ -402,14 +411,14 @@ int cap_enable(__u32 caps) {
 }
 
 int cap_drop(ctx_t *ctx_p, __u32 caps) {
-	debug(1, "Dropping all Linux capabilites but 0x%x", caps);
+	debug(1, "Dropping all Linux capabilities but 0x%x", caps);
 
 	struct __user_cap_header_struct	cap_hdr = {0};
 	struct __user_cap_data_struct	cap_dat = {0};
 
 	cap_hdr.version = _LINUX_CAPABILITY_VERSION;
 	if (capget(&cap_hdr, &cap_dat) < 0) {
-		error_or_debug((ctx_p->flags[CAP_PRESERVE] != CAP_PRESERVE_TRY) ? -1 : 3, "Cannot get capabilites with capget()");
+		error_or_debug((ctx_p->flags[CAP_PRESERVE] != CAP_PRESERVE_TRY) ? -1 : 3, "Cannot get capabilities with capget()");
 		return errno;
 	}
 	debug(3, "old: cap.eff == 0x%04x; cap.prm == 0x%04x; cap.inh == 0x%04x.",
@@ -443,13 +452,14 @@ int cap_drop(ctx_t *ctx_p, __u32 caps) {
 }
 
 #endif
-int __privileged_kill_child_itself(pid_t child_pid, int signal) {
+int __privileged_kill_child_itself(pid_t child_pid, int signal, char ignoreerrors) {
 	// Checking if it's a child
 	if (waitpid(child_pid, NULL, WNOHANG)>=0) {
 		debug(3, "Sending signal %u to child process with pid %u.",
 			signal, child_pid);
 		if (kill(child_pid, signal)) {
-			error("Got error while kill(%u, %u)", child_pid, signal);
+			if (!ignoreerrors)
+				error("Got error while kill(%u, %u)", child_pid, signal);
 			return errno;
 		}
 
@@ -526,21 +536,25 @@ int privileged_execvp_check_arguments(struct pa_options *opts, const char *u_fil
 		argv       = args[a_i].v;
 		isexpanded = args[a_i].isexpanded;
 
-		// Checking the number of arguments
+		debug(8, "Checking the number of arguments: %i <> %i", argc, u_argc);
 		if (argc != u_argc)
 			continue;
 
 		critical_on (!argc);
 
-		// Checking the execution file
-		if (pa_strcmp(argv[0], u_file, isexpanded[0]))
-			continue;
+		debug(8, "Checking the execution file: \"%s\" <> \"%s\"; isexpanded == %i", argv[0], u_file, isexpanded[0]);
+		if (pa_strcmp(argv[0], u_file, isexpanded[0])) {
+			debug(1, "The file to be executed didn't match (argv[0] != u_file): \"%s\" != \"%s\"", argv[0], u_file);
+			break;
+		}
 
-		// Checking arguments
+		debug(8, "Checking arguments");
 		i = 1;
 		while (i < argc) {
-			if (pa_strcmp(argv[i], u_argv[i], isexpanded[i]))
+			if (pa_strcmp(argv[i], u_argv[i], isexpanded[i])) {
+				debug(1, "An argument #%i didn't match (argv[%i] != u_argv[%i]): \"%s\" != \"%s\"", argv[i], argv[i]);
 				break;
+			}
 			i++;
 		}
 
@@ -576,6 +590,7 @@ int privileged_execvp_check_arguments(struct pa_options *opts, const char *u_fil
 		}
 	}
 
+	debug(1, "a_i == %i; SHARGS_MAX == %i; u_argc == %i", SHARGS_MAX, a_i, u_argc);
 	critical("Arguments are wrong. This should happend only on hacking attack.");
 	return EPERM;
 }
@@ -813,7 +828,10 @@ int privileged_handler(ctx_t *ctx_p)
 {
 # ifdef READWRITE_SIGNALLING
 	char buf[1] = {0};
+# else
+	struct timespec wait_timeout = {0};
 # endif
+
 	int setup = 0;
 	uid_t exec_uid = 65535;
 	gid_t exec_gid = 65535;
@@ -883,16 +901,36 @@ int privileged_handler(ctx_t *ctx_p)
 		if (!hl_lock_p->enabled || !hl_wait(
 			HLLOCK_HANDLER
 #  ifdef HL_LOCK_TRIES_AUTO
-			, (long)hl_lock_p->tries
+			, hl_lock_p->tries[HLLOCK_HANDLER]
 #  endif
 		)) {
 			if (opts->isprocsplitting)
 				critical_on(!parent_isalive());
 # endif
 # ifdef READWRITE_SIGNALLING
+#  warning		READWRITE_SIGNALLING can cause process hanging on clsync shutdown
 			read_inf(priv_read_fd, buf, 1);
 # else
-			critical_on (pthread_cond_wait(pthread_cond_privileged_p, pthread_mutex_privileged_p));
+			int rc;
+			while (1) {
+				rc = pthread_cond_timedwait(pthread_cond_privileged_p, pthread_mutex_privileged_p, &wait_timeout);
+				if (!rc)
+					break;
+				if (rc != ETIMEDOUT)
+					critical("Got error while pthread_cond_timedwait()");
+				debug(10, "pthread_cond_timedwait() timed out");
+
+				if (opts->isprocsplitting)
+					exit_on(!parent_isalive());
+
+				{
+					debug(20, "Resetting wait_timeout");
+					struct timeval now;
+					gettimeofday(&now, NULL);
+					wait_timeout.tv_sec	= now.tv_sec  + SLEEP_SECONDS;
+					wait_timeout.tv_nsec	= now.tv_usec * 1000;
+				}
+			}
 # endif
 # ifdef HL_LOCKS
 			if (hl_lock_p->enabled)
@@ -904,7 +942,7 @@ int privileged_handler(ctx_t *ctx_p)
 			mprotect((void *)cmd_ret_p,	sizeof(*cmd_ret_p),	PROT_WRITE);
 		}
 
-		debug(10, "Got command %u", cmd_p->action);
+		debug(10, "Got command %u (euid:egid => %i:%i)", cmd_p->action, geteuid(), getegid());
 
 		if (!setup && cmd_p->action != PA_SETUP)
 			critical("A try to use commands before PA_SETUP");
@@ -918,6 +956,7 @@ int privileged_handler(ctx_t *ctx_p)
 				critical_on(pa_setup(opts, cmd_p->arg.ctx_p, &exec_uid, &exec_gid));
 				mprotect(opts, sizeof(*opts), PROT_READ);
 				use_args_check = cmd_p->arg.ctx_p->flags[CHECK_EXECVP_ARGS];
+				cap_drop(ctx_p, ctx_p->caps);	// TODO: Find out why "permission denined" without this line
 				setup++;
 				critical_on(errno);
 				break;
@@ -1016,8 +1055,16 @@ int privileged_handler(ctx_t *ctx_p)
 						error("Cannot fork().");
 						break;
 					case  0:
-						debug(4, "setgid(%u) == %i", exec_gid, setgid(exec_gid));
-						debug(4, "setuid(%u) == %i", exec_uid, setuid(exec_uid));
+#ifdef ANTIPARANOID
+						if (ctx_p->privileged_gid != exec_gid)
+#endif
+							debug(4, "setgid(%u) == %i", exec_gid, setgid(exec_gid));
+
+#ifdef ANTIPARANOID
+						if (ctx_p->privileged_uid != exec_uid)
+#endif
+							debug(4, "setuid(%u) == %i", exec_uid, setuid(exec_uid));
+
 						debug(3, "execvp(\"%s\", argv)", file);
 						exit(execvp(file, argv));
 				}
@@ -1028,7 +1075,7 @@ int privileged_handler(ctx_t *ctx_p)
 			case PA_KILL_CHILD: {
 				debug(20, "PA_KILL_CHILD");
 				struct pa_kill_child_arg *arg_p = (void *)&cmd_p->arg.kill_child;
-				cmd_ret_p->ret = (void *)(long)__privileged_kill_child_itself(arg_p->pid, arg_p->signal);
+				cmd_ret_p->ret = (void *)(long)__privileged_kill_child_itself(arg_p->pid, arg_p->signal, arg_p->ignoreerrors);
 				break;
 			}
 # ifdef CGROUP_SUPPORT
@@ -1543,12 +1590,13 @@ int __privileged_fork_setuid_execvp_threadsplit(
 	return (long)ret;
 }
 
-int __privileged_kill_child_wrapper(pid_t pid, int signal)
+int __privileged_kill_child_wrapper(pid_t pid, int signal, char ignoreerrors)
 {
 	void *ret = (void *)(long)-1;
 
-	cmd_p->arg.kill_child.pid    = pid;
-	cmd_p->arg.kill_child.signal = signal;
+	cmd_p->arg.kill_child.pid          = pid;
+	cmd_p->arg.kill_child.signal       = signal;
+	cmd_p->arg.kill_child.ignoreerrors = ignoreerrors;
 
 	privileged_action(
 # ifdef HL_LOCK_TRIES_AUTO
@@ -1680,11 +1728,6 @@ int privileged_init(ctx_t *ctx_p)
 #endif
 
 #ifdef CAPABILITIES_SUPPORT
-# ifdef HL_LOCKS
-	if (ncpus == 1)
-		hl_shutdown(HLLOCK_HANDLER);
-# endif
-
 	if (ctx_p->flags[SPLITTING] == SM_OFF) {
 #endif
 
@@ -1747,7 +1790,9 @@ int privileged_init(ctx_t *ctx_p)
 
 	SAFE ( pthread_mutex_lock(pthread_mutex_runner_p),		return errno;);
 
+# ifdef UNSHARE_SUPPORT
 	unshare(CLONE_NEWIPC);
+# endif
 
 	switch (ctx_p->flags[SPLITTING]) {
 		case SM_THREAD: {
@@ -1783,20 +1828,57 @@ int privileged_init(ctx_t *ctx_p)
 
 			// Running the privileged helper
 			SAFE ( (helper_pid = fork_helper()) == -1,	return errno);
-			if (!helper_pid)
+			if (!helper_pid) {
+				if (ctx_p->privileged_gid != ctx_p->gid) {
+//					SAFE ( cap_enable(CAP_TO_MASK(CAP_SETGID)), return errno; );
+
+					debug(3, "[privileged] Trying to set real gid to %i (ctx_p->privileged_gid)", ctx_p->privileged_gid);
+					SAFE( setgid(ctx_p->privileged_gid),	return errno);
+				}
+
+				if (ctx_p->privileged_uid != ctx_p->uid) {
+//					SAFE ( cap_enable(CAP_TO_MASK(CAP_SETUID)), return errno; );
+
+					debug(3, "[privileged] Trying to set real uid to %i (ctx_p->privileged_uid)", ctx_p->privileged_uid);
+					SAFE( setuid(ctx_p->privileged_uid),	return errno);
+				}
+
 				exit(privileged_handler(ctx_p));
+			}
 			break;
 		}
 		default:
 			critical("Invalid ctx_p->flags[SPLITTING]: %i", ctx_p->flags[SPLITTING]);
 	}
+
+	if (ctx_p->flags[GID] || ctx_p->flags[UID])
+		SAFE( seteuid(0), {error("Not enough permission to start a privileged thread/fork"); return errno;});
+
+	if (ctx_p->flags[GID]) {
+		SAFE( setegid(0), return errno);
+		debug(3, "[non-privileged] Trying to drop real gid %i (ctx_p->gid)", getgid(), ctx_p->gid);
+		SAFE( setgid(ctx_p->gid), return errno );
+	}
+
+	if (ctx_p->flags[UID]) {
+		debug(3, "[non-privileged] Trying to drop real uid %i (ctx_p->uid)", getuid(), ctx_p->uid);
+		SAFE( setuid(ctx_p->uid), return errno );
+	}
+
+# ifdef HL_LOCKS
+	if (ncpus == 1)
+		hl_shutdown(HLLOCK_HANDLER);
+# endif
+
 	critical_on(!helper_isalive());
 
+# ifdef UNSHARE_SUPPORT
 	// The rest routines
 	if (ctx_p->flags[DETACH_NETWORK] == DN_NONPRIVILEGED) {
 		SAFE ( cap_enable(CAP_TO_MASK(CAP_SYS_ADMIN)),	return errno; );
 		SAFE ( unshare(CLONE_NEWNET),			return errno; );
 	}
+# endif
 	SAFE ( cap_drop(ctx_p, 0),				return errno; );
 
 	debug(4, "Waiting for the privileged thread to get prepared");
@@ -1876,9 +1958,12 @@ int privileged_deinit(ctx_t *ctx_p)
 		}
 		case SM_PROCESS: {
 			int status;
-			__privileged_kill_child_itself(helper_pid, SIGKILL);
-			debug(9, "waitpid(%u, ...)", helper_pid);
-			waitpid(helper_pid, &status, 0);
+			if (!ctx_p->flags[SECCOMP_FILTER]) {
+				if (!__privileged_kill_child_itself(helper_pid, SIGKILL, 1)) {
+					debug(9, "waitpid(%u, ...)", helper_pid);
+					waitpid(helper_pid, &status, 0);
+				}
+			}
 			shm_free((void *)cmd_p);
 			shm_free((void *)cmd_ret_p);
 # ifdef HL_LOCKS
